@@ -1,26 +1,27 @@
-"fit.ghypmv" <- function(data, lambda=1,alpha.bar = 1,
-  mu = NULL,sigma=NULL,gamma=NULL,
-  opt.pars = c(lambda=T, alpha.bar = T,mu=T,sigma=T,gamma=!symmetric),
-  symmetric=F, nit=2000, reltol=1e-10, abstol=reltol*10, 
-  na.rm=F,silent=FALSE, save.data=T,...)
+"fit.ghypmv" <- function(data, lambda = 1, alpha.bar = 1,
+                         mu = NULL, sigma = NULL, gamma = NULL,
+                         opt.pars = c(lambda = T, alpha.bar = T, mu = T, sigma = T, gamma = !symmetric),
+                         symmetric = F, standardize = F, nit = 2000, reltol = 1e-8, abstol = reltol * 10, 
+                         na.rm = F, silent = FALSE, save.data = T,...)
 {
+  call <- match.call(expand.dots = TRUE) 
 
-  data <- check.data(data=data,case="mv",na.rm=na.rm)
+  data <- check.data(data = data, case = "mv", na.rm = na.rm)
   n <- nrow(data)
   d <- ncol(data)
 
-  tmp.abar2chipsi <- abar2chipsi(alpha.bar,lambda)
+  tmp.abar2chipsi <- abar2chipsi(alpha.bar, lambda)
   chi <- tmp.abar2chipsi$chi
   psi <- tmp.abar2chipsi$psi
   #<----- check parameters of the mixing distribution for consistency --------->
-  check.gig.pars(lambda,chi,psi)
+  check.gig.pars(lambda, chi, psi)
   #<----------------- check opt.pars  for consistency ------------------------->
-  opt.pars <- check.opt.pars(opt.pars)
+  opt.pars <- check.opt.pars(opt.pars, symmetric)
   if(is.null(mu)){
     mu <- colMeans(data)
   }
   if(is.null(gamma) | symmetric){
-    gamma <- rep(0,d)
+    gamma <- rep(0, d)
   }
   if(symmetric){
     opt.pars["gamma"] <- FALSE
@@ -28,61 +29,78 @@
   if(is.null(sigma)){
     sigma <- var(data)
   }
-  
+
+  #<--------- check normal and skewness parameters  for consistency ----------->
+  check.norm.pars(mu, sigma, gamma, d)
+
+  if(standardize){
+    # data will be standardized and initial values will be adapted
+    tmp.mean <- colMeans(data)
+    tmp.var <- var(data)
+
+    sigma.chol <- t(chol(solve(tmp.var)))
+    
+    data.backup <- data    
+    data <- apply(data.backup, MARGIN = 2, FUN = function(x){x - mean(x)})
+    data <-  data %*% sigma.chol
+    
+    sigma <- t(sigma.chol) %*% sigma %*% sigma.chol
+    gamma <- as.vector(sigma.chol %*% gamma)
+    mu <- as.vector(sigma.chol %*% (mu - tmp.mean))
+  }  
+
   #<------- loglikelihood function of the inverse gamma distribution ---------->
-  t.optfunc <- function(thepars,delta.sum,xi.sum,n.rows){
-      nu <- max(2 * abs(thepars),2+sqrt(.Machine$double.eps))
-      term1 <- -n.rows*nu*log(nu/2-1)/2
-      term2 <- (nu/2+1)*xi.sum+(nu/2-1)*delta.sum
-      term3 <- n.rows*lgamma(nu/2)
-      out <- term1+term2+term3
-      out
+  t.optfunc <- function(thepars, delta.sum, xi.sum, n.rows){
+      nu <- -2 * t.transform(thepars)
+      term1 <- -n.rows * nu * log(nu/2 - 1)/2
+      term2 <- (nu/2 + 1) * xi.sum + (nu/2 - 1) * delta.sum
+      term3 <- n.rows * lgamma(nu/2)
+      out <- term1 + term2 + term3
+      return(out)
   }
 
   #<------------ loglikelihood function of the gamma distribution ------------->
-  vg.optfunc <- function(thepars,xi.sum,eta.sum,n.rows){
-      thepars <- max(abs(thepars),sqrt(.Machine$double.eps))
-      term1 <- n.rows*(thepars*log(thepars)-lgamma(thepars))
-      term2 <- (thepars-1)*xi.sum-thepars*eta.sum
-      out <- -(term1+term2)
-      out
+  vg.optfunc <- function(thepars, xi.sum, eta.sum, n.rows){
+      thepars <- exp(thepars)
+      term1 <- n.rows * (thepars * log(thepars) - lgamma(thepars))
+      term2 <- (thepars - 1) * xi.sum - thepars * eta.sum
+      out <- -(term1 + term2)
+      return(out)
   }
   
   #<- loglikelihood function of the generalized inverse gaussian distribution ->
   gig.optfunc <- function(thepars, mix.pars.fixed, delta.sum, eta.sum, xi.sum, n.rows)
   {
     out <- NA
-    tmp.pars <- c(thepars,mix.pars.fixed)
+    tmp.pars <- c(thepars, mix.pars.fixed)
     lambda <- tmp.pars["lambda"]
-    alpha.bar <- abs(tmp.pars["alpha.bar"])
-    tmp.abar2chipsi <- abar2chipsi(alpha.bar,lambda)
+    alpha.bar <- exp(tmp.pars["alpha.bar"])
+    tmp.abar2chipsi <- abar2chipsi(alpha.bar, lambda)
     chi <- tmp.abar2chipsi$chi
     psi <- tmp.abar2chipsi$psi
     if (lambda < 0 & psi == 0){                             # t
-     out <-  t.optfunc(lambda,delta.sum,xi.sum,n.rows)
+     out <-  t.optfunc(lambda, delta.sum, xi.sum, n.rows)
     }else if(lambda > 0 & chi == 0) {                       # VG
-        out <-  vg.optfunc(lambda,xi.sum,eta.sum,n.rows)
+        out <-  vg.optfunc(lambda, xi.sum, eta.sum, n.rows)
      }else{                                                 # ghyp, hyp, NIG
-        term1 <- (lambda-1)*xi.sum
-        term2 <- -chi*delta.sum/2
-        term3 <- -psi*eta.sum/2
-        term4 <- -n.rows*lambda*log(chi)/2+ n.rows*lambda*log(psi)/2 -
-                 n.rows * besselM3(lambda, sqrt(chi*psi), logvalue=T)
-        out <- -( term1+term2+term3+term4 )
+        term1 <- (lambda - 1) * xi.sum
+        term2 <- -chi * delta.sum/2
+        term3 <- -psi * eta.sum/2
+        term4 <- -n.rows * lambda * log(chi)/2 + n.rows * lambda * log(psi)/2 -
+                  n.rows * besselM3(lambda, sqrt(chi * psi), logvalue=T)
+        out <- -(term1 + term2 + term3 + term4)
     }
-    out
+    return(out)
   }
-  #<--------- check normal and skewness parameters  for consistency ----------->
-  check.norm.pars(mu,sigma,gamma)
-
-  ll <-  sum(internal.dghypmv(data,lambda=lambda,chi=chi,psi=psi,
-             mu=mu, sigma=sigma,gamma= gamma, logvalue=T ))
              
   #<------------------------- Initialize fitting loop ------------------------->
   i <- 0
   rel.closeness <- 100
   abs.closeness <- 100
-  tmp.fit <- list(convergence=0,message=NULL)
+  tmp.fit <- list(convergence = 0, message = NULL)
+  
+  ll <-  sum(internal.dghypmv(data, lambda = lambda, chi = chi, psi = psi,
+                              mu = mu, sigma = sigma, gamma = gamma, logvalue=T))  
   #<------------------------- Start interations ------------------------------->
   while ((abs.closeness > abstol) & (rel.closeness > reltol) & (i < nit)){
     i <- i+1
@@ -116,28 +134,32 @@
     Q <- mahalanobis(data, mu, inv.sigma, inverted = TRUE)
     Offset <- t(gamma) %*% inv.sigma %*% gamma
     xi.sum <- sum(Egig(lambda-d/2,Q+chi,psi+Offset,func="logx", check.pars = FALSE))
-    if(alpha.bar==0 & lambda > 0 & !opt.pars["alpha.bar"] & opt.pars["lambda"]){
+    if(alpha.bar==0 & lambda > 0 & !opt.pars["alpha.bar"] & opt.pars["lambda"]){ # VG case
       eta.sum <- sum(Egig(lambda-d/2,Q+chi,psi+Offset,func="x", check.pars = FALSE))
-      tmp.fit <- suppressWarnings(optim(lambda, vg.optfunc, eta.sum=eta.sum, 
+      
+      tmp.fit <- suppressWarnings(optim(log(lambda), vg.optfunc, eta.sum=eta.sum, 
                                         xi.sum=xi.sum, n.rows=n,...))
-      lambda <- abs(tmp.fit$par)                   
-    }else if(alpha.bar==0 & lambda < 0 & !opt.pars["alpha.bar"] & opt.pars["lambda"]){
+      lambda <- exp(tmp.fit$par)                   
+    }else if(alpha.bar==0 & lambda < 0 & !opt.pars["alpha.bar"] & opt.pars["lambda"]){ # Student-t case
       delta.sum <- sum(Egig(lambda-d/2,Q+chi,psi+Offset,func="1/x", check.pars = FALSE))
-      tmp.fit <- suppressWarnings(optim(lambda, t.optfunc, delta.sum=delta.sum, 
+      tmp.fit <- suppressWarnings(optim(inv.t.transform(lambda), t.optfunc, delta.sum=delta.sum, 
                                         xi.sum=xi.sum, n.rows=n,...))
-      lambda <- -abs(tmp.fit$par)    
+      lambda <- t.transform(tmp.fit$par)    
     }else if(opt.pars["lambda"] | opt.pars["alpha.bar"]){  ## ghyp, hyp, NIG case
       delta.sum <- sum(Egig(lambda-d/2,Q+chi,psi+Offset,func="1/x", check.pars = FALSE))
       eta.sum <- sum(Egig(lambda-d/2,Q+chi,psi+Offset,func="x", check.pars = FALSE))
-      mix.pars <- c(lambda=unname(lambda),alpha.bar=unname(alpha.bar))
+      
+      mix.pars <- c(lambda=unname(lambda),alpha.bar=log(unname(alpha.bar)))
       opt.pars.mix <- opt.pars[c("lambda","alpha.bar")]
       thepars <- mix.pars[opt.pars.mix]
+      
       tmp.fit <- suppressWarnings(optim(thepars, gig.optfunc, 
                                         mix.pars.fixed=mix.pars[!opt.pars.mix], 
                                         delta.sum=delta.sum,eta.sum=eta.sum, 
                                         xi.sum=xi.sum, n.rows=n,...))
+      
       lambda <- c(tmp.fit$par,mix.pars[!opt.pars.mix])["lambda"]
-      alpha.bar <- abs(c(tmp.fit$par,mix.pars[!opt.pars.mix])["alpha.bar"])
+      alpha.bar <- exp(c(tmp.fit$par,mix.pars[!opt.pars.mix])["alpha.bar"])
     }
     tmp.abar2chipsi <- abar2chipsi(alpha.bar,lambda)
     chi <- tmp.abar2chipsi$chi
@@ -155,9 +177,9 @@
     }
     #<--------------------------- Print result -------------------------------->
     if(!silent){   
-      message <- paste("It ",i,"; rel Closeness ",sprintf("%.6 E",rel.closeness),
-        "; log Likelihood ",sprintf("%.6 E",ll),"; alpha.bar ",sprintf("%.6 E",alpha.bar),
-        "; lambda ",sprintf("%.6 E",lambda),sep="")
+      message <- paste("iter: ",i,"; rel.closeness: ",sprintf("% .6E",rel.closeness),
+        "; log-likelihood: ",sprintf("% .6E",ll),"; alpha.bar: ",sprintf("% .6E",alpha.bar),
+        "; lambda: ",sprintf("% .6E",lambda),sep="")
       print(message)
     }
   }      
@@ -172,13 +194,23 @@
   }
 
   converged <- F
-  if(i<nit & is.finite(rel.closeness) & is.finite(abs.closeness)){
+  if(i < nit & is.finite(rel.closeness) & is.finite(abs.closeness)){
     converged <- T
   }
 
-  if(save.data==T){
-     data <- data
-  }else{
+  if(standardize){
+    data <- data.backup 
+    inv.sigma.chol <- solve(sigma.chol)
+    mu <- as.vector(inv.sigma.chol %*% mu + tmp.mean)
+    sigma <- t(inv.sigma.chol) %*% sigma %*% inv.sigma.chol
+    gamma <- as.vector(inv.sigma.chol %*% gamma)
+    abar.chi.psi <- abar2chipsi(alpha.bar = alpha.bar, lambda = lambda)
+    ll <- sum(internal.dghypmv(x = data, lambda = lambda, chi = abar.chi.psi$chi,
+                               psi = abar.chi.psi$psi, mu = mu, sigma = sigma, 
+                               gamma = gamma, logvalue = TRUE))
+  }
+
+  if(!save.data){
     data <- NULL
   }
 
@@ -187,11 +219,15 @@
                               d/2 * (d + 1) * opt.pars[c("sigma")])
   aic <- -2 * ll + 2 * nbr.fitted.params 
   
-  tmp.ghyp.object <- ghyp(lambda=lambda,alpha.bar=alpha.bar,
-                          mu=mu,sigma=sigma,gamma=gamma,data=data)
-
-  return(fit.ghyp(tmp.ghyp.object,llh=ll,n.iter=i,converged=converged,
-                  error.code=conv, error.message=conv.type,
+  ghyp.object <- ghyp(lambda = lambda, alpha.bar = alpha.bar,
+                      mu = mu, sigma = sigma, gamma = gamma, data = data)
+  
+  ghyp.object@parametrization = "lambda.alpha.bar"
+  ghyp.object@call = call 
+  
+  return(fit.ghyp(ghyp.object, llh = ll, n.iter = i, converged = converged,
+                  error.code = conv, error.message = conv.type,
                   fitted.params = opt.pars, aic = aic))
 }
 
+
